@@ -10,18 +10,17 @@ module Text.Email.Parser
     )
 where
 
-import Control.Applicative
-import Control.Monad (void)
-
+import           Control.Applicative
+import           Control.Monad (void)
+import           Data.Attoparsec.ByteString.Char8
+import           Data.ByteString (ByteString)
 import qualified Data.ByteString.Char8 as BS
-import Data.ByteString (ByteString)
-
-import Data.Attoparsec.ByteString.Char8
-
-import Data.Data (Data, Typeable)
-import GHC.Generics (Generic)
+import           Data.Data (Data, Typeable)
+import           GHC.Generics (Generic)
 import qualified Text.Read as Read
 
+import           Text.Domain.Parser (domainParser)
+
 -- | Represents an email address.
 data EmailAddress = EmailAddress ByteString ByteString
     deriving (Eq, Ord, Data, Typeable, Generic)
@@ -58,21 +57,25 @@ domainPart (EmailAddress _ d) = d
 
 -- | A parser for email addresses.
 addrSpec :: Parser EmailAddress
-addrSpec = do
-    l <- local
-    _ <- char '@'
-    d <- domain
-    return (EmailAddress l d)
+addrSpec = unsafeEmailAddress <$> local <* char '@' <*> domain
 
 local :: Parser ByteString
 local = dottedAtoms
 
 domain :: Parser ByteString
-domain = dottedAtoms <|> domainLiteral
+domain = domainName <|> domainLiteral
+
+domainName :: Parser ByteString
+domainName = do
+    raw <- BS.append <$> dottedAtoms <*> option BS.empty (string (BS.pack "."))
+    case parseOnly (domainParser <* endOfInput) raw of
+        Left err -> fail err
+        Right result -> return result
 
 dottedAtoms :: Parser ByteString
 dottedAtoms = BS.intercalate (BS.singleton '.') <$>
-        between1 (optional cfws) (atom <|> quotedString) `sepBy1` char '.'
+        between1 (optional cfws)
+            (atom <|> quotedString) `sepBy1` char '.'
 
 atom :: Parser ByteString
 atom = takeWhile1 isAtomText
@@ -91,8 +94,8 @@ isDomainText x = inClass "\33-\90\94-\126" x || isObsNoWsCtl x
 
 quotedString :: Parser ByteString
 quotedString =
-    (\x -> BS.concat [BS.singleton '"', BS.concat x, BS.singleton '"']) <$>
-        between (char '"') (char '"')
+    (BS.cons '"' . flip BS.snoc '"' . BS.concat) <$>
+        between1 (char '"')
             (many (optional fws >> quotedContent) <* optional fws)
 
 quotedContent :: Parser ByteString
