@@ -11,12 +11,10 @@ module Text.Email.Validate
         )
 where
 
-import           Control.Applicative        ((<*))
-
 import           Data.Attoparsec.ByteString (endOfInput, parseOnly)
 import           Data.ByteString            (ByteString)
 
-import           Text.Email.Parser          (EmailAddress(..), addrSpec, domainPart,
+import           Text.Email.Parser          (EmailAddress, addrSpec, domainPart,
                                              localPart, toByteString, unsafeEmailAddress)
 
 -- | Smart constructor for an email address
@@ -36,4 +34,4 @@ isValid = either (const False) (const True) . validate
 -- | If you want to find out *why* a particular string is not
 --   an email address, use this.
 validate :: ByteString -> Either String EmailAddress
-validate = parseOnly (addrSpec <* endOfInput)
+validate = parseOnly (addrSpec >>= \r -> endOfInput >> return r)
